"""Product blocks for :class:`GeantIP` products."""

from orchestrator.domain.base import ProductBlockModel
from orchestrator.types import SubscriptionLifecycle
from pydantic import Field

from gso.products.product_blocks.edge_port import EdgePortBlock, EdgePortBlockInactive, EdgePortBlockProvisioning
from gso.utils.shared_enums import APType


class NRENAccessPortInactive(
    ProductBlockModel, lifecycle=[SubscriptionLifecycle.INITIAL], product_block_name="NRENAccessPort"
):
    """An access port for an R&E :term:`NREN` service that is inactive."""

    nren_ap_type: APType | None = None
    geant_ip_ep_list: list[EdgePortBlockInactive] = Field(default_factory=list)


class NRENAccessPortProvisioning(NRENAccessPortInactive, lifecycle=[SubscriptionLifecycle.PROVISIONING]):
    """An access port for an R&E :term:`NREN` service that is being provisioned."""

    nren_ap_type: APType
    geant_ip_ep_list: list[EdgePortBlockProvisioning]  # type: ignore[assignment]


class NRENAccessPort(NRENAccessPortProvisioning, lifecycle=[SubscriptionLifecycle.ACTIVE]):
    """An access port for an R&E :term:`NREN` service."""

    #: The type of Access Port
    nren_ap_type: APType
    #: The list of Edge Ports where this service terminates.
    geant_ip_ep_list: list[EdgePortBlock]  # type: ignore[assignment]


class GeantIPBlockInactive(
    ProductBlockModel, lifecycle=[SubscriptionLifecycle.INITIAL], product_block_name="GeantIPBlock"
):
    """A GÉANT IP subscription that is currently inactive. See :class:`GeantIPBlock`."""

    geant_ip_ap_list: list[NRENAccessPortInactive] = Field(default_factory=list)


class GeantIPBlockProvisioning(GeantIPBlockInactive, lifecycle=[SubscriptionLifecycle.PROVISIONING]):
    """A GÉANT IP subscription that is currently being provisioned. See :class:`GeantIPBlock`."""

    geant_ip_ap_list: list[NRENAccessPortProvisioning]  # type: ignore[assignment]


class GeantIPBlock(GeantIPBlockProvisioning, lifecycle=[SubscriptionLifecycle.ACTIVE]):
    """A GÉANT IP subscription block."""

    #: The list of Access Points where this service is present.
    geant_ip_ap_list: list[NRENAccessPort]  # type: ignore[assignment]
